''***************************************
''*         Stepper Motor Control 3.0   *
''***************************************

''* 
''*  ------------------------------------------  *
''*  Version 3.0

''*  Now calls PulseGenerator2 which correctly handles an odd # of pulses
''*  in free-running mode.
''*
''*  Two-pin step & dir stepper motor driver
''* 
''*  Methods:
''*     .Start(StepPin, DirPin)
''*     .Stop

CON

  MinPeriod  = 4000  '50 uS   20,000 pulses/sec  @ 8000 pulses/rev = 2.5 rev/sec
  FixedWidth = 2000  '25 uS
  
    
VAR

  long  PulsesPerRev
    
  long  StepPin
  long  DirPin
  
  long  DirSign   '+1 when moving CW,  -1 when moving CCW
   
OBJ

   PG : "PulseGenerator2"   ' running in separate cog
   
  'debugging only 
  'serial       : "SerialMirror2"             '1 Cog 
                
 
PUB Init( pStepPin, pDirPin) : okay

   'Set Default Value:
    PulsesPerRev := 2000    'motor    = 200 steps/rev
                            'gecko201 = 10 pulses per motor step 
                            'gear ratio wheel-to-motor: 4:1   

    StepPin := pStepPin     'save Step & Dir pin assignments
    DirPin  := pDirPin

    dira[DirPin]~~   'set:   _DirPin pin set as an output
    outa[DirPin]~    'clear: _DirPin pin logic level set low.
       
    okay := PG.Start(StepPin, 10)   'start( OutputPin, TriggerPin)

    'note:  As pulse generator is utilized in a mode which does not require
    '       a trigger pin, pin#10 is arbitrarily passed as the TriggerPin parameter.
    '       It will be ignored in 'free running' mode.

    PG.Units(-1)  'Width & Period Units Are Now Measured in Clock Cycles
    
    PG.Mode(0)       ' disabled initially

   'Default rotation speed : 1 rev per second
   '------------------------------------------
   '10 Pulses/Step * 200 Steps/MotorRev * (4 MotorRev/ 1 WheelRev)  = 8000 pulses/WheelRev [DEFAULT]
   'With the values below (2000 pulses/sec), default wheel rotation rate: 1 revolution per 4 seconds.
    PG.Period(80000)        ' 1000 uS period (interval between pulses) for 1000 pulses per second
    PG.Width( FixedWidth )  ' 25 uS fixed pulse width
    
    PG.Number(0)     ' default at startup 
    
PUB halt

    PG.Mode(0)  'disable motor
    PG.Write_Parameters 
     
PUB reset

    Init(StepPin, DirPin)
 
    
PUB DistToTarget_Pulses

   'return the remaining pulses to be sent in the current pulse pattern
    return PG.RemainingPulses 

PUB DistToTarget_Degrees: pulses_remaining

   'return the degrees of movement remaining .......
    return PG.RemainingPulses * 360 / PulsesPerRev 

                                        
PUB SetDir_CW

    outa[DirPin]~    'clear: _DirPin pin logic level set low.
    DirSign := +1

PUB SetDir_CCW

    outa[DirPin]~~   'set:   _DirPin pin logic level set high.
    DirSign := -1

PUB SetDist_Pulses (pulses)

    PG.Number(pulses)

PUB SetSpeed_DegsPerSec( DegsPerSec ) | CntsPerPulse

    '1 rev/sec @ 8000 PulsesPerRev = 8000 pulses per second

    '1 sec = 80_000_000 clock counts

    '80_000_000 cnts/sec / 8000 pps  =  10000  counts/pulse 

    'clkfreq = 80_000_000

    'counts/pulse = clkfreq / ((degsPerSec/360) * PulsesPerRev)
                 '= ((clkfreq/PulsesPerRev) * 360 )/DegsPerSec

     CntsPerPulse := ((clkfreq/PulsesPerRev) * 360)/DegsPerSec

     'serial.str(string(13,"Degs/Sec = "))
     'serial.dec(DegsPerSec)
     'serial.putc(13)

     'serial.str(string("Cnts/Pulse = "))
     'serial.dec(CntsPerPulse)
     'serial.putc(13)

     if CntsPerPulse < MinPeriod
        PG.Period(MinPeriod)
     else
        PG.Period(CntsPerPulse)   
            
    'PG.Period(10000)    

PUB SetPulsesPerRev( ppr)

    PulsesPerRev := ppr

PUB GetPulsesPerRev

    return PulsesPerRev

PUB SetDist_Angle (degrees) | pulses

   'Upper limit set by resolution of 32-bit integer arithmetic
   'If PulsesPerRev = 8000, 'degrees' parameter must be less
   'than ~268,000 degs (~745 revolutions) to prevent overflow.

    pulses := (degrees * PulsesPerRev)/360
    PG.Number(pulses)
        
PUB Move

    PG.Mode(1)             'enable free-running mode
    PG.Write_Parameters

   'PG.ShowVariables
   